// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

// 项目自身文件
#include "X86Generator.h"
#include "DDLog.h"

using namespace DDLog;

X86Generator::X86Generator()
{
    qCDebug(appLog) << "X86Generator constructor";
}
