// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

// 项目自身文件
#include "ArmGenerator.h"

#include <QDebug>

ArmGenerator::ArmGenerator()
{
    qDebug() << "ArmGenerator constructor";
}
