/****************************************************************************
** Copyright (c) 2013-2014 Debao Zhang <hello@debao.me>
** All right reserved.
**
** Permission is hereby granted, free of charge, to any person obtaining
** a copy of this software and associated documentation files (the
** "Software"), to deal in the Software without restriction, including
** without limitation the rights to use, copy, modify, merge, publish,
** distribute, sublicense, and/or sell copies of the Software, and to
** permit persons to whom the Software is furnished to do so, subject to
** the following conditions:
**
** The above copyright notice and this permission notice shall be
** included in all copies or substantial portions of the Software.
**
** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
** MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
** NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
** LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
** OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
** WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**
****************************************************************************/
#include "xlsxsimpleooxmlfile_p.h"
#include <QIODevice>

namespace QXlsx {
SimpleOOXmlFile::SimpleOOXmlFile(CreateFlag flag)
    :AbstractOOXmlFile(flag)
{
}

void SimpleOOXmlFile::saveToXmlFile(QIODevice *device) const
{
    device->write(xmlData);
}

QByteArray SimpleOOXmlFile::saveToXmlData() const
{
    return xmlData;
}

bool SimpleOOXmlFile::loadFromXmlData(const QByteArray &data)
{
    xmlData = data;
    return true;
}

bool SimpleOOXmlFile::loadFromXmlFile(QIODevice *device)
{
    xmlData = device->readAll();
    return true;
}

}
