// SPDX-FileCopyrightText: 2022 lpxxn <mi_duo@live.com>
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef CONSTANTS_H
#define CONSTANTS_H

namespace Docx {
namespace Constants {

const char OPC_CORE_PROPERTIES[]    = "application/vnd.openxmlformats-package.core-properties+xml";
const char WML_DOCUMENT_MAIN[]      = "application/vnd.openxmlformats-officedocument.wordprocessingml.document.main+xml";
const char WML_NUMBERING[]          = "application/vnd.openxmlformats-officedocument.wordprocessingml.numbering+xml";
const char WML_STYLES[]             = "application/vnd.openxmlformats-officedocument.wordprocessingml.styles+xml";
const char IMAGE[]                  = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/image";
const char EXTERNAL[]               = "External";
const char  INTERNAL[]              = "Internal";
const char OFFICE_DOCUMENT[]        = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/officeDocument";

// content Types values
const char CONTENT_TYPES_URI[]      = "[Content_Types].xml";
const char PML_PRINTER_SETTINGS[]   = "application/vnd.openxmlformats-officedocument.presentationml.printerSettings";
const char SML_PRINTER_SETTINGS[]   = "application/vnd.openxmlformats-officedocument.spreadsheetml.printerSettings";
const char WML_PRINTER_SETTINGS[]   = "application/vnd.openxmlformats-officedocument.wordprocessingml.printerSettings";
const char BMP[]                    = "image/bmp";
const char X_EMF[]                  = "image/x-emf";
const char X_FONTDATA[]             = "application/x-fontdata";
const char GIF[]                    = "image/gif";
const char JPEG[]                   = "image/jpeg";
const char PNG[]                    = "image/png";
const char OPC_RELATIONSHIPS[]      = "http://schemas.openxmlformats.org/package/2006/relationships";
const char TIFF[]                   = "image/tiff";
const char MS_PHOTO[]               = "image/vnd.ms-photo";
const char X_WMF[]                  = "image/x-wmf";
const char SML_SHEET[]              = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";
const char XML[]                    = "application/xml";



}
}

#endif // CONSTANTS_H
