/****************************************************************************
** Meta object code from reading C++ file 'controlinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../deepin-devicemanager/deepin-devicemanager-server/deepin-devicecontrol/src/controlinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'controlinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16ControlInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto ControlInterface::qt_create_metaobjectdata<qt_meta_tag_ZN16ControlInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ControlInterface",
        "D-Bus Interface",
        "org.deepin.DeviceControl",
        "sigUpdate",
        "",
        "getRemoveInfo",
        "getAuthorizedInfo",
        "enable",
        "hclass",
        "name",
        "path",
        "value",
        "enable_device",
        "strDriver",
        "enablePrinter",
        "enableKeyboard",
        "vid",
        "pid",
        "sPath",
        "disableOutDevice",
        "devInfo",
        "disableInDevice",
        "isDeviceEnabled",
        "unique_id",
        "setWakeupMachine",
        "wakeup",
        "setNetworkWake",
        "logicalName",
        "updateWakeup",
        "isNetworkWakeup",
        "setMonitorWorkingDBFlag",
        "flag",
        "monitorWorkingDBFlag"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sigUpdate'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'getRemoveInfo'
        QtMocHelpers::SlotData<QString()>(5, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'getAuthorizedInfo'
        QtMocHelpers::SlotData<QString()>(6, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'enable'
        QtMocHelpers::SlotData<bool(const QString &, const QString &, const QString &, const QString &, bool, const QString)>(7, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
            { QMetaType::Bool, 12 }, { QMetaType::QString, 13 },
        }}),
        // Slot 'enable'
        QtMocHelpers::SlotData<bool(const QString &, const QString &, const QString &, const QString &, bool)>(7, 4, QMC::AccessPublic | QMC::MethodCloned | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'enablePrinter'
        QtMocHelpers::SlotData<bool(const QString &, const QString &, const QString &, bool)>(14, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 }, { QMetaType::Bool, 12 },
        }}),
        // Slot 'enableKeyboard'
        QtMocHelpers::SlotData<bool(const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, bool, const QString)>(15, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 },
            { QMetaType::QString, 18 }, { QMetaType::QString, 11 }, { QMetaType::Bool, 12 }, { QMetaType::QString, 13 },
        }}),
        // Slot 'disableOutDevice'
        QtMocHelpers::SlotData<void(const QString &)>(19, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'disableInDevice'
        QtMocHelpers::SlotData<void()>(21, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'isDeviceEnabled'
        QtMocHelpers::SlotData<bool(const QString &)>(22, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 23 },
        }}),
        // Slot 'setWakeupMachine'
        QtMocHelpers::SlotData<bool(const QString &, const QString &, bool)>(24, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 23 }, { QMetaType::QString, 10 }, { QMetaType::Bool, 25 },
        }}),
        // Slot 'setNetworkWake'
        QtMocHelpers::SlotData<bool(const QString &, bool)>(26, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 27 }, { QMetaType::Bool, 25 },
        }}),
        // Slot 'updateWakeup'
        QtMocHelpers::SlotData<void(const QString &)>(28, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'isNetworkWakeup'
        QtMocHelpers::SlotData<int(const QString &)>(29, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Int, {{
            { QMetaType::QString, 27 },
        }}),
        // Slot 'setMonitorWorkingDBFlag'
        QtMocHelpers::SlotData<void(bool)>(30, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Slot 'monitorWorkingDBFlag'
        QtMocHelpers::SlotData<bool()>(32, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<ControlInterface, qt_meta_tag_ZN16ControlInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject ControlInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusService::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ControlInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ControlInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16ControlInterfaceE_t>.metaTypes,
    nullptr
} };

void ControlInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ControlInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigUpdate(); break;
        case 1: { QString _r = _t->getRemoveInfo();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { QString _r = _t->getAuthorizedInfo();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->enable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->enable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->enablePrinter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->enableKeyboard((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[8])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->disableOutDevice((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->disableInDevice(); break;
        case 9: { bool _r = _t->isDeviceEnabled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->setWakeupMachine((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->setNetworkWake((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->updateWakeup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: { int _r = _t->isNetworkWakeup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->setMonitorWorkingDBFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: { bool _r = _t->monitorWorkingDBFlag();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ControlInterface::*)()>(_a, &ControlInterface::sigUpdate, 0))
            return;
    }
}

const QMetaObject *ControlInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ControlInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ControlInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QDBusService::qt_metacast(_clname);
}

int ControlInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusService::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void ControlInterface::sigUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
