/****************************************************************************
** Meta object code from reading C++ file 'PageDriverManager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../deepin-devicemanager/deepin-devicemanager/src/Page/PageDriverManager.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PageDriverManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17PageDriverManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto PageDriverManager::qt_create_metaobjectdata<qt_meta_tag_ZN17PageDriverManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PageDriverManager",
        "startScanning",
        "",
        "itemClicked",
        "itemStr",
        "scanInfo",
        "info",
        "progress",
        "scanFinished",
        "ScanResult",
        "sr",
        "sigBackupFinished",
        "bsuccess",
        "scanDriverInfo",
        "slotDriverOperationClicked",
        "index",
        "itemIndex",
        "DriverOperationItem::Mode",
        "mode",
        "slotItemCheckedClicked",
        "checked",
        "slotBackupItemCheckedClicked",
        "slotDownloadProgressChanged",
        "msg",
        "slotDownloadFinished",
        "slotInstallProgressChanged",
        "slotInstallProgressFinished",
        "err",
        "slotInstallAllDrivers",
        "slotBackupAllDrivers",
        "slotScanFinished",
        "slotUndoInstall",
        "slotUndoBackup",
        "slotListViewWidgetItemClicked",
        "slotBackupProgressChanged",
        "slotBackupFinished",
        "slotRestoreProgress",
        "strDeatils",
        "slotRestoreFinished",
        "success"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'startScanning'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'itemClicked'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'scanInfo'
        QtMocHelpers::SignalData<void(const QString &, int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::Int, 7 },
        }}),
        // Signal 'scanFinished'
        QtMocHelpers::SignalData<void(ScanResult)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'sigBackupFinished'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'scanDriverInfo'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDriverOperationClicked'
        QtMocHelpers::SlotData<void(int, int, DriverOperationItem::Mode)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::Int, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotItemCheckedClicked'
        QtMocHelpers::SlotData<void(int, bool)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'slotBackupItemCheckedClicked'
        QtMocHelpers::SlotData<void(int, bool)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'slotDownloadProgressChanged'
        QtMocHelpers::SlotData<void(QStringList)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 23 },
        }}),
        // Slot 'slotDownloadFinished'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotInstallProgressChanged'
        QtMocHelpers::SlotData<void(int)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'slotInstallProgressFinished'
        QtMocHelpers::SlotData<void(bool, int)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 12 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'slotInstallAllDrivers'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotBackupAllDrivers'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotScanFinished'
        QtMocHelpers::SlotData<void(ScanResult)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotUndoInstall'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUndoBackup'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotListViewWidgetItemClicked'
        QtMocHelpers::SlotData<void(const QString &)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'slotBackupProgressChanged'
        QtMocHelpers::SlotData<void(int)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'slotBackupFinished'
        QtMocHelpers::SlotData<void(bool)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'slotRestoreProgress'
        QtMocHelpers::SlotData<void(int, const QString &)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::QString, 37 },
        }}),
        // Slot 'slotRestoreFinished'
        QtMocHelpers::SlotData<void(bool, const QString &)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 39 }, { QMetaType::QString, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PageDriverManager, qt_meta_tag_ZN17PageDriverManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PageDriverManager::staticMetaObject = { {
    QMetaObject::SuperData::link<DWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PageDriverManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PageDriverManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17PageDriverManagerE_t>.metaTypes,
    nullptr
} };

void PageDriverManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PageDriverManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->startScanning(); break;
        case 1: _t->itemClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->scanInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->scanFinished((*reinterpret_cast<std::add_pointer_t<ScanResult>>(_a[1]))); break;
        case 4: _t->sigBackupFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->scanDriverInfo(); break;
        case 6: _t->slotDriverOperationClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<DriverOperationItem::Mode>>(_a[3]))); break;
        case 7: _t->slotItemCheckedClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 8: _t->slotBackupItemCheckedClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->slotDownloadProgressChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 10: _t->slotDownloadFinished(); break;
        case 11: _t->slotInstallProgressChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->slotInstallProgressFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 13: _t->slotInstallAllDrivers(); break;
        case 14: _t->slotBackupAllDrivers(); break;
        case 15: _t->slotScanFinished((*reinterpret_cast<std::add_pointer_t<ScanResult>>(_a[1]))); break;
        case 16: _t->slotUndoInstall(); break;
        case 17: _t->slotUndoBackup(); break;
        case 18: _t->slotListViewWidgetItemClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 19: _t->slotBackupProgressChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->slotBackupFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->slotRestoreProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 22: _t->slotRestoreFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PageDriverManager::*)()>(_a, &PageDriverManager::startScanning, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageDriverManager::*)(const QString & )>(_a, &PageDriverManager::itemClicked, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageDriverManager::*)(const QString & , int )>(_a, &PageDriverManager::scanInfo, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageDriverManager::*)(ScanResult )>(_a, &PageDriverManager::scanFinished, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageDriverManager::*)(bool )>(_a, &PageDriverManager::sigBackupFinished, 4))
            return;
    }
}

const QMetaObject *PageDriverManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageDriverManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PageDriverManagerE_t>.strings))
        return static_cast<void*>(this);
    return DWidget::qt_metacast(_clname);
}

int PageDriverManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void PageDriverManager::startScanning()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PageDriverManager::itemClicked(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void PageDriverManager::scanInfo(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void PageDriverManager::scanFinished(ScanResult _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void PageDriverManager::sigBackupFinished(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
