/****************************************************************************
** Meta object code from reading C++ file 'PageTableHeader.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../deepin-devicemanager/deepin-devicemanager/src/Page/PageTableHeader.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PageTableHeader.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15PageTableHeaderE_t {};
} // unnamed namespace

template <> constexpr inline auto PageTableHeader::qt_create_metaobjectdata<qt_meta_tag_ZN15PageTableHeaderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PageTableHeader",
        "itemClicked",
        "",
        "row",
        "refreshInfo",
        "exportInfo",
        "enableDevice",
        "enable",
        "uninstallDriver",
        "installDriver",
        "wakeupMachine",
        "wakeup",
        "signalCheckPrinterStatus",
        "bool&",
        "isPrinter",
        "isInstalled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'itemClicked'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'refreshInfo'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'exportInfo'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enableDevice'
        QtMocHelpers::SignalData<void(int, bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Bool, 7 },
        }}),
        // Signal 'uninstallDriver'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'installDriver'
        QtMocHelpers::SignalData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'wakeupMachine'
        QtMocHelpers::SignalData<void(int, bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Bool, 11 },
        }}),
        // Signal 'signalCheckPrinterStatus'
        QtMocHelpers::SignalData<void(int, bool &, bool &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 13, 14 }, { 0x80000000 | 13, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PageTableHeader, qt_meta_tag_ZN15PageTableHeaderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PageTableHeader::staticMetaObject = { {
    QMetaObject::SuperData::link<DWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PageTableHeaderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PageTableHeaderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15PageTableHeaderE_t>.metaTypes,
    nullptr
} };

void PageTableHeader::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PageTableHeader *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->refreshInfo(); break;
        case 2: _t->exportInfo(); break;
        case 3: _t->enableDevice((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->uninstallDriver((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->installDriver((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->wakeupMachine((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->signalCheckPrinterStatus((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PageTableHeader::*)(int )>(_a, &PageTableHeader::itemClicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageTableHeader::*)()>(_a, &PageTableHeader::refreshInfo, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageTableHeader::*)()>(_a, &PageTableHeader::exportInfo, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageTableHeader::*)(int , bool )>(_a, &PageTableHeader::enableDevice, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageTableHeader::*)(int )>(_a, &PageTableHeader::uninstallDriver, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageTableHeader::*)(int )>(_a, &PageTableHeader::installDriver, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageTableHeader::*)(int , bool )>(_a, &PageTableHeader::wakeupMachine, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageTableHeader::*)(int , bool & , bool & )>(_a, &PageTableHeader::signalCheckPrinterStatus, 7))
            return;
    }
}

const QMetaObject *PageTableHeader::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageTableHeader::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PageTableHeaderE_t>.strings))
        return static_cast<void*>(this);
    return DWidget::qt_metacast(_clname);
}

int PageTableHeader::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void PageTableHeader::itemClicked(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PageTableHeader::refreshInfo()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void PageTableHeader::exportInfo()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void PageTableHeader::enableDevice(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void PageTableHeader::uninstallDriver(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void PageTableHeader::installDriver(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void PageTableHeader::wakeupMachine(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void PageTableHeader::signalCheckPrinterStatus(int _t1, bool & _t2, bool & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
