/****************************************************************************
** Meta object code from reading C++ file 'DBusDriverInterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../deepin-devicemanager/deepin-devicemanager/src/DriverControl/DBusDriverInterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'DBusDriverInterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19DBusDriverInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto DBusDriverInterface::qt_create_metaobjectdata<qt_meta_tag_ZN19DBusDriverInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DBusDriverInterface",
        "processChange",
        "",
        "value",
        "details",
        "processEnd",
        "sucess",
        "msg",
        "downloadProgressChanged",
        "downloadFinished",
        "installProgressChanged",
        "progress",
        "installProgressFinished",
        "bsuccess",
        "err",
        "installFinished",
        "installProgressDetail",
        "backupProgressFinished",
        "slotProcessChange",
        "detail",
        "slotProcessEnd",
        "success",
        "slotCallFinished",
        "QDBusPendingCallWatcher*",
        "watcher",
        "slotDownloadProgressChanged",
        "slotDownloadFinished",
        "slotInstallProgressChanged",
        "slotInstallProgressFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'processChange'
        QtMocHelpers::SignalData<void(qint32, QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'processEnd'
        QtMocHelpers::SignalData<void(bool, QString)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'downloadProgressChanged'
        QtMocHelpers::SignalData<void(QStringList)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 7 },
        }}),
        // Signal 'downloadFinished'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'installProgressChanged'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'installProgressFinished'
        QtMocHelpers::SignalData<void(bool, int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 }, { QMetaType::Int, 14 },
        }}),
        // Signal 'installFinished'
        QtMocHelpers::SignalData<void(bool, QString)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'installProgressDetail'
        QtMocHelpers::SignalData<void(int, QString)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'backupProgressFinished'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotProcessChange'
        QtMocHelpers::SlotData<void(qint32, const QString &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 19 },
        }}),
        // Slot 'slotProcessEnd'
        QtMocHelpers::SlotData<void(bool, const QString &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 21 }, { QMetaType::QString, 7 },
        }}),
        // Slot 'slotCallFinished'
        QtMocHelpers::SlotData<void(QDBusPendingCallWatcher *)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'slotDownloadProgressChanged'
        QtMocHelpers::SlotData<void(const QStringList &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 7 },
        }}),
        // Slot 'slotDownloadFinished'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotInstallProgressChanged'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'slotInstallProgressFinished'
        QtMocHelpers::SlotData<void(bool, int)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 13 }, { QMetaType::Int, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DBusDriverInterface, qt_meta_tag_ZN19DBusDriverInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DBusDriverInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DBusDriverInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DBusDriverInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19DBusDriverInterfaceE_t>.metaTypes,
    nullptr
} };

void DBusDriverInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DBusDriverInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->processChange((*reinterpret_cast<std::add_pointer_t<qint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->processEnd((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->downloadProgressChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 3: _t->downloadFinished(); break;
        case 4: _t->installProgressChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->installProgressFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->installFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->installProgressDetail((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->backupProgressFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->slotProcessChange((*reinterpret_cast<std::add_pointer_t<qint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 10: _t->slotProcessEnd((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->slotCallFinished((*reinterpret_cast<std::add_pointer_t<QDBusPendingCallWatcher*>>(_a[1]))); break;
        case 12: _t->slotDownloadProgressChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 13: _t->slotDownloadFinished(); break;
        case 14: _t->slotInstallProgressChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->slotInstallProgressFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusPendingCallWatcher* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DBusDriverInterface::*)(qint32 , QString )>(_a, &DBusDriverInterface::processChange, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusDriverInterface::*)(bool , QString )>(_a, &DBusDriverInterface::processEnd, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusDriverInterface::*)(QStringList )>(_a, &DBusDriverInterface::downloadProgressChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusDriverInterface::*)()>(_a, &DBusDriverInterface::downloadFinished, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusDriverInterface::*)(int )>(_a, &DBusDriverInterface::installProgressChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusDriverInterface::*)(bool , int )>(_a, &DBusDriverInterface::installProgressFinished, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusDriverInterface::*)(bool , QString )>(_a, &DBusDriverInterface::installFinished, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusDriverInterface::*)(int , QString )>(_a, &DBusDriverInterface::installProgressDetail, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusDriverInterface::*)(bool )>(_a, &DBusDriverInterface::backupProgressFinished, 8))
            return;
    }
}

const QMetaObject *DBusDriverInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DBusDriverInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DBusDriverInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DBusDriverInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void DBusDriverInterface::processChange(qint32 _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void DBusDriverInterface::processEnd(bool _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void DBusDriverInterface::downloadProgressChanged(QStringList _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DBusDriverInterface::downloadFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void DBusDriverInterface::installProgressChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void DBusDriverInterface::installProgressFinished(bool _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void DBusDriverInterface::installFinished(bool _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void DBusDriverInterface::installProgressDetail(int _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void DBusDriverInterface::backupProgressFinished(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
